// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum (gir tilgang til en rekke fields (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"
 
/**
 * <strong>Description:</strong><br>
 * Abstract bulbs to be used togehter with the abstract class <code>Stand</code> to make an hourglass.<br>
 * Supplies all necessary common methods for its subclasses.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
 
public abstract class Bulbs implements GLEnum
{
	// VARIABLER SOM IKKE FOREKOMMER I SUBKLASSENE
	// Variabler for materialer
	MaterialFactory matFactory; // Brukes til  instansiere riktig klasse av materiale
	Material bulbsMaterial; // Glassets materiale
	Material sandMaterial; // Sandens materiale
	Material pointMaterial; // Bare for metoden testMarkCurve()
	// Bestemmer hvor mange kontrollpunkter Bezierflaten skal ha  
	int UN = 5; // Antall kontrollpunkter langs X-aksen
	int VN = 5; // Antall kontrollpunkter langs Y-aksen
	// Faktorer for radiuskompensasjon
	float rf = 1.65f;
	float rf1 = 0.75f;
	// Representerer et gluNewQuadric object
	long sandPart; 
	
	// VARIABLER SOM KOMMER SOM PARAMETERE TIL KONSTRUKTREN
	GLFunc gl = null; // Interface for OpenGL "function mapping"
	GLUFunc glu = null; //Interface for GLU "function mapping"
	GLUTFunc glut = null; // Interface for GLUT "function mapping"
	
	// VARIABLER SOM BLIR SATT VED INITIERING AV BULBS (kunne vel ogs kommet som parameter til konstruktren)
	String strBulbsMaterial;
	String strSandMaterial;
	
	// VARIABLER SOM INSTANSIERES I SUBKLASSENE
	// Glassets type, (Classic, Modern eller Bizarre)
	String bulbsTypeName;
	// Lengden p halve timeglasset, fra midten og nedover/oppover (hele glasset 6.0f)
	float bulbheight;
	//Plassering p Y-aksen
	float ctrz0; 	// ClassicBulbs, Y = 0
	float ctrz1; 	// ClassicBulbs, Y = 0.3
	float ctrz2; 	// ClassicBulbs, Y = 0.9
	float ctrz3;	// ClassicBulbs, Y = 1.5
	float ctrz4;	    	// ClassicBulbs, Y = 3
	// Lengden p radiene til selve glasset, fra midten av timeglasset og nedover/oppover
	// bRad0 og bRad4 blir lik radien til timeglasset p det aktuelle stedet
	// bRad1, bRad2 og bRad3 er radier til kontrollpunkter, ikke til selve timeglasset
	// Plassering p X-aksen
	float bRad0; // startpunkt, ClassicBulbs, 0.02f*bulbheight
	float bRad1; //                 ClassicBulbs, 0.02f*bulbheight
	float bRad2; //                 ClassicBulbs, 0.4f*bulbheight
	float bRad3; //                 ClassicBulbs, 0.7f*bulbheight
	float bRad4; // sluttpunkt, ClassicBulbs, 0.5f*bulbheight
	// "Tykkelsen" p glasset
	float glassThickness;
	// Lengden p radiene til sanden, fra midten av timeglasset og nedover/oppover
	// Plassering p X-aksen
	float sRad0; //startpunkt
	float sRad1;
	float sRad2;
	float sRad3;
	float sRad4; //sluttpunkt
	// Matrisearray med alle kontrollpunktene til glasset
	float bulbCtrPoints [];
	// Matrisearray med alle kontrollpunktene til sanden
	float sandCtrPoints [];
	// 5 arrayer som inneholder kontrollpunktene til sandens Bezierkurve (opp langs glasset)
	// Brukes til  regne ut X-Y-Z-verdier for et punkt p sandens kurve
	float sandBezierCurvePoints0[]; // startpunkt
	float sandBezierCurvePoints1[];
	float sandBezierCurvePoints2[];
	float sandBezierCurvePoints3[];
	float sandBezierCurvePoints4[]; // endepunkt
	
	// VARIABLER FOR SANDEN I DEN NEDERSTE DELEN AV GLASSET
	// Det hyeste sandnivet
	float maxLowerSandLevel; // ClassicBulbs = 0.1f 
	// Utganspunkt for  sette radien til den lile haugen (lowerTinyHeap)
	float lowerTinyHeapRadiusStart; 
	// Hvor mange steg som behves fr lowerTinyHeap nr glassets vegger
	int lowerTinyHeapSteps;
	// Hvor mange steg som behves for Beziersanden  n maxLowerSandLevel
	int bottomLowerSandSteps;
	
	// VARIABLER FOR SANDEN I DEN VERSTE DELEN AV GLASSET
	// Det hyeste sandnivet
	float maxUpperSandLevel; // ClassicBulbs = 0.8f
	// Disse 4 variablene bestemmer med hvor mye sanden skal synke
	float upperSandShrink1;
	float upperSandShrink2;
	float upperSandShrink3;
	float upperSandShrink4;
	// Disse 4 variablene regulerer hvor mange steg som skal tas
	// med  en gitt upperSandShrink-verdi 
	int upperSandSteps1;
	int upperSandSteps2;
	int upperSandSteps3;
	int upperSandSteps4;
	
    
	/**
	* Bulbs constructor
	* @param gl the <code>GLFunc</code> interface that is used for this component
	* @param glu the <code>GLUFunc</code> interface that is used for this component
	* @param glut the <code>GLUTFunc</code> interface that is used for this component
	*/
	public Bulbs( GLFunc gl, GLUFunc glu, GLUTFunc glut) 
	{
		//System.out.println("Fra konstruktren i BULBS");
		this.gl = gl;
		this.glu = glu;
		this.glut = glut;
		matFactory = new MaterialFactory (gl);
	}
	
	/**
	* Sets all the variables particular to the subclass.<br>
	* All subclasses must implement this method.
	*/
	public abstract void setSpecialBulbValues();
	
	/**
	* Draws the glass sand container (bulbs) of the hourglass
	*/
	public void drawBulbs()
	{
		// Aktiviserer materialet til glasset
		bulbsMaterial.enableMaterial(GL_FRONT_AND_BACK);
        
		// Lager Bezierflaten
		gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, UN, 0.0f, 1.0f, 3*UN, VN, bulbCtrPoints);
		// Aktiviserer Bezierflaten
		gl.glEnable(GL_MAP2_VERTEX_3);
		// Stiller inn antall masker i "rutenettet"
		gl.glMapGrid2f(40, 0.0f, 1.0f, 30, 0.0f, 1.0f);
		
		// Bytter om p forside og bakside p glasset, gjr jeg ikke dette fr ikke glasset noen glans
		gl.glFrontFace(GL_CW);
        
		// Roterer p plass og tegner ut de 4 delene timeglasset bestr av
		gl.glPushMatrix();
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // verst foran
		gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // verst bak
		gl.glRotatef(180.0f, 0.0f, 0.0f, 1.0f);
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // Nederst bak
		gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // Nederst foran
		gl.glPopMatrix();
		
		// Setter hva som er forside og bakside tilbake til defaultsetting
		gl.glFrontFace(GL_CCW);
		
		// Pasifiserer brukt materiale
		bulbsMaterial.disableMaterial();
	}
	
	/**
	* Draws the sand in the lower part of the hourglass
	* @param needsBottomLowerSand indicates if the sand should go up along the walls
	* @param trickle indicates if the sandtrickle should be drawn
	* @param lowerT a value indicating a point on the curve
	* @param lowerTinyHeapRadius the radius for lowerTinyHeap
	*/ 
	public void drawLowerSand(boolean needsBottomLowerSand, boolean trickle, float lowerT, float lowerTinyHeapRadius)
	{
		// Henter X og Y -verdiene til lowerT
		// Sender kurven inn den "gale" veien for  f lowerT = 1 til  vre fullt nedre glass
		float lowerTPoint[] = 
			getPointOnCurve( sandBezierCurvePoints4,  sandBezierCurvePoints3, 
			sandBezierCurvePoints2, sandBezierCurvePoints1, sandBezierCurvePoints0, lowerT);
		// Y-verdien til lowerT blir brukt til  sette hvor p Y-aksen klippeplanet skal vre, 
		// (er ogs "grunnplan" for konen p Beziersanden)
		float lowerCutoff = - lowerTPoint[1];
		//System.out.println("  lowerCutoff er: " + lowerCutoff);
		// X-verdien er radien til Beziersanden ved klippeplanet (radien til konen p Beziersanden)
		float lowerTPointRadius = lowerTPoint[0];
		
		//float lowerTPointZValue = lowerTPoint[2]; 
		//System.out.println("  lowerT er: " + lowerT);
		//System.out.println("  lowerTPointRadius er: " + lowerTPointRadius);
		//System.out.println("  lowerTPointZValue er: " + lowerTPointZValue);
		//testMarkCurve(lowerTPoint[0], lowerTPoint[1], lowerTPoint[2]);
		
		// Radius for bunnplaten av sandhaugen
		float lowerSandBottomDiskRadius;
		
		// Hvis sanden skal g oppover "veggene" brukes bezierflatene til timeglasset som utganspunkt,
		// s klippes bezierflatene med klippeplanet
		if (needsBottomLowerSand)
		{
			// Definerer og aktiviserer det nederste klippeplanet,
			// skal vise fram det som er p den negative siden av Y aksen, fra lowerCutoff
			double lowerCutplane[] = {0.0, -1.0, 0.0, (double)lowerCutoff};
			gl.glClipPlane(GL_CLIP_PLANE0, lowerCutplane);
			gl.glEnable(GL_CLIP_PLANE0);
			
			// Lager Bezierflaten
			gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,UN,0.0f,1.0f,3*UN,VN,sandCtrPoints);
			// Aktiviserer Bezierflaten
			gl.glEnable(GL_MAP2_VERTEX_3);
			// Stiller inn antall masker i "rutenettet"
			gl.glMapGrid2f(40, 0.0f, 1.0f, 30, 0.0f, 1.0f);
			
			// Roterer p plass og tegner de 2 delene sanden bestr av
			gl.glPushMatrix();
			gl.glRotatef(180.0f, 0.0f, 0.0f, 1.0f);
			gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // Nederst foran
			gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
			gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // Nederst bak
			gl.glPopMatrix();
			
			// Pasifiserer brukt klippeplan           
			gl.glDisable(GL_CLIP_PLANE0); 
						
			// Toppen av haugen etter at sanden har "mtt veggen"
			gl.glPushMatrix();
			gl.glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
			gl.glTranslatef(0.0f, 0.0f, lowerCutoff);
			glut.glutSolidCone (lowerTPointRadius, lowerTPointRadius * 0.6f, 80, 1); // "slices" m vre det dobbelte av maskeantallet i u-retningen
			gl.glPopMatrix();
			
			// Setter radien til "bunnplaten" av sandhaugen
			lowerSandBottomDiskRadius = getEndSandRadius(); 
			//System.out.println("  lowerSandBottomDiskRadius er: " + lowerSandBottomDiskRadius);
		}
		// Hvis sanden ikke har mtt veggen enda
		else
		{	
			// Setter radien til "bunnplaten" av sandhaugen
			lowerSandBottomDiskRadius = lowerTinyHeapRadius; 
			//Tegner lowerTinyHea) 
			gl.glPushMatrix();
			gl.glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
			gl.glTranslatef(0.0f, 0.0f, lowerCutoff);
			glut.glutSolidCone (lowerTinyHeapRadius, lowerTinyHeapRadius * 0.6f, 80, 1); // "slices" m vre det dobbelte av maskeantallet i u-retningen
			gl.glPopMatrix();
		}
		
		// Sanden m f en bunnflate ( lowerSandBottomDisk) fordi bunnen av glasset kan vre laget av glass
		//Bunnflaten m vre litt over stttens flate, ellers "blander" de seg
		sandPart = glu.gluNewQuadric(); // Initierer et "tegneobjekt"
		gl.glPushMatrix();
		gl.glTranslatef(0.0f, -bulbheight + 0.005f, 0.0f); 
		gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		glu.gluDisk(sandPart, 0.0, lowerSandBottomDiskRadius, 80, 1); // "slices" m vre det dobbelte av maskeantallet i u-retningen
		gl.glPopMatrix();
		
		// Tegner "strle" bare nr det behves
		// M trekke strlen litt opp fra bunnen for at den ikke skal sees gjennom bunnflaten til sanden - 0.005f
		if(trickle)
		{
			gl.glPushMatrix();
			gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
			glu.gluCylinder(sandPart, 0.02f, 0.02f, bulbheight - 0.005f, 10, 1); 
			gl.glPopMatrix(); 
		}
		glu.gluDeleteQuadric(sandPart); // Sletter "tegneobjektet" nr jeg ikke trenger det lenger
	}
	
	/**
	* Draws the sand in the upper part of the hourglass
	* @param upperT a value indicating a point on the curve
	*/ 
	public void drawUpperSand(float upperT)
	{
		
		// Henter X og Y -verdiene til upperT, upperT = 1 er fullt vre glass
		float upperTPoint[] = 
			getPointOnCurve( sandBezierCurvePoints0, sandBezierCurvePoints1, 
			sandBezierCurvePoints2, sandBezierCurvePoints3, sandBezierCurvePoints4, upperT);
			
		// Y-verdien til upperT blir brukt til  sette hvor p Y-aksen klippeplanet skal vre, 
		// (er ogs "grunnplan" for "lokket" p Beziersanden)
		float upperCutoff =  upperTPoint[1];
		
		// X-verdien er radien til Beziersanden ved klippeplanet (radien til "lokket" p Beziersanden)
		float upperTPointRadius = upperTPoint[0];
			
		// Definerer og aktiviserer det vre klippeplanet
		// Plassering: P den positive Y-aksen....
		double upperCutplane[] = {0.0, -1.0, 0.0, (double)upperCutoff};
		gl.glClipPlane(GL_CLIP_PLANE1, upperCutplane);
		gl.glEnable(GL_CLIP_PLANE1);
		
		// Lager Bezierflaten
		gl.glMap2f(GL_MAP2_VERTEX_3,0.0f,1.0f,3,UN,0.0f,1.0f,3*UN,VN,sandCtrPoints);
		// Aktiviserer Bezierflaten
		gl.glEnable(GL_MAP2_VERTEX_3);
		// Stiller inn antall masker i "rutenettet"
		gl.glMapGrid2f(40, 0.0f, 1.0f, 30, 0.0f, 1.0f);
		
		// Roterer p plass og tegner de 2 delene sanden bestr av 
		gl.glPushMatrix();
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // verst foran
		gl.glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		gl.glEvalMesh2(GL_FILL, 0, 40, 0, 30); // verst bak
		gl.glPopMatrix();
		
		// Pasifiserer brukt clipplane
		gl.glDisable(GL_CLIP_PLANE1);
		
		// Tegner lokk p sanden
		sandMaterial.enableMaterial(GL_FRONT_AND_BACK); //lyset m kunne virke p innsiden av konen (det er den som synes)
		gl.glPushMatrix();
		gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		gl.glTranslatef(0.0f, 0.0f, -upperCutoff);
		glut.glutSolidCone (upperTPointRadius, 0.1f, 80, 1); // "slices" m vre det dobbelte av maskeantallet i u-retningen
		gl.glPopMatrix();
		sandMaterial.enableMaterial(GL_FRONT);
	}
    
    /**
     * Draws a sphere to symbolize the sand while the hourglass is turning
     */
	public void drawSandWhileTurning()
	{
		gl.glPushMatrix();
		gl.glTranslatef(0.0f, -1.7f, 0.0f);
		glut.glutSolidSphere (0.8f, 40, 40);
		gl.glPopMatrix(); 
	}
	
	/**
     * This function returns an X-Y-Z point along the curve, depending on t (0 to 1)
     * @param p0, p1, p2, p3, p4 arrays containing the Bezier-curves controlpoints, 
     * where p0 is the startpoint of the curve and p4 the endpoint of the curve .
     * @param t the step along the curve (ranging from 0 to 1)
     * @return curvePoint an array containing the XYZ values of the point along the curve
     * curvePoint[0] = X value, curvePoint[1] = Y value, curvePoint[2] = Z value, 
     */
	public float[] getPointOnCurve(float[] p0, float[] p1, float[] p2, float[] p3, float[] p4, float t)
	{
		// Array som skal inneholde X, Y og Z verdiene til det nskede punktet
		float curvePoint[] = new float [3]; 
		
		// Lager noen variabler for  forkorte uttrykk. Dette hjelper for oversikten 
		// (og sparer mye utregning senere)
		float omT = 1.0f - t; // (1 - t)
		float omTi2 = omT * omT; // (1 - t)^2
		float omTi3 = omT * omT * omT; // (1 - t)^3
		float omTi4 = omT * omT * omT * omT; // (1 - t)^4
		float ti2 = t * t; // t^2
		float ti3 = t * t * t; // t^3
		float ti4 = t * t * t * t; // t^4
		
		// Formelen for  finne et punkt p kurven
		/*curvePoint 	= p0 * omTi4   
					+ p1 * 4 * omTi3 * t  
					+ p2 * 6 * omTi2 * ti2  
					+ p3 * 4 * omT * ti3     
					+ p4 * ti4;*/
				
		// X-verdiene
		curvePoint[0] 	= p0[0] * omTi4   
					+ p1[0] * 4 * omTi3 * t  
					+ p2[0] * 6 * omTi2 * ti2  
					+ p3[0] * 4 * omT * ti3     
					+ p4[0] * ti4;
		// Y-verdiene
		curvePoint[1] = p0[1] * omTi4   
					+ p1[1] * 4 * omTi3 * t  
					+ p2[1] * 6 * omTi2 * ti2  
					+ p3[1] * 4 * omT * ti3     
					+ p4[1] * ti4;
		// Z-verdiene
		curvePoint[2] = p0[2] * omTi4   
					+ p1[2] * 4 * omTi3 * t  
					+ p2[2] * 6 * omTi2 * ti2  
					+ p3[2] * 4 * omT * ti3     
					+ p4[2] * ti4;
		
		// Returnerer arrayen med X, Y og Z verdiene til det nskede punktet
		return curvePoint;				
	}
	
	/**
     * Draws lines between the controlpoints for one of the bulbs Bezier surfaces.<br>
     * Base code snatch from Brre Stenseths tramp.java.
     */
	public void drawBulbCtrPointLines()
	{
		gl.glLineWidth(1);
		// Svart farge til linjene
		float ambR[]={0.0f, 0.0f, 0.0f, 1.0f};
		float diffR[]={0.0f, 0.0f, 0.0f, 1.0f};
		float specR[]={0.0f, 0.0f, 0.0f, 1.0f};
		float shineR=32.00f;
		gl.glMaterialfv(GL_FRONT,GL_AMBIENT,ambR);
		gl.glMaterialfv(GL_FRONT,GL_DIFFUSE,diffR);
		gl.glMaterialfv(GL_FRONT,GL_SPECULAR,specR);
		gl.glMaterialf(GL_FRONT,GL_SHININESS,shineR);
		
		// Tellere
		int u,v;
		int step = 3 * UN; // 3-tallet betyr at kontrollpunktarrayene er tredimensjonale
		
		for(u=0;u<UN;u++)
		{
			gl.glBegin(GL_LINE_STRIP);
			for(v=0;v<VN;v++)
			{
				gl.glVertex3f(  bulbCtrPoints[v*step+u*3+0],
					bulbCtrPoints[v*step+u*3+1],
					bulbCtrPoints[v*step+u*3+2]);
			}
			gl.glEnd();
		}
		
		for(v=0;v<VN;v++)
		{
			gl.glBegin(GL_LINE_STRIP);	
			for(u=0;u<UN;u++)
			{
				gl.glVertex3f(  bulbCtrPoints[v*step+u*3+0],
					bulbCtrPoints[v*step+u*3+1],
					bulbCtrPoints[v*step+u*3+2]);
			}
			gl.glEnd();
		}
	}
	
	public void activateSandMaterial ( )
	{
		// Aktiviserer materialet til sanden
		sandMaterial.enableMaterial(GL_FRONT);
	}
	
	public void deactivateSandMaterial ( )
	{
		// Pasifiserer materialet til sanden
		sandMaterial.disableMaterial();
	}
	
	/**
     * Gets the start radius of the tiny sand heap
     * @return the start radius
     */
	public float getLowerTinyHeapRadiusStart ( )
	{
		return lowerTinyHeapRadiusStart;
	}
	
	/**
     * Gets the number of steps needed for the 
     * lowerTinyHeapRadius to reach the sandEndRadius (sRad4)
     * @return  lowerTinyHeapSteps the number of steps needed
     */
	public int getLowerTinyHeapSteps ( )
	{
		return lowerTinyHeapSteps;
	}
	
	/**
     * Gets the number of steps needed for the 
     * sand to grow in the lower part
     * @return bottomLowerSandSteps the number of steps needed
     */
	public int getBottomLowerSandSteps ( )
	{
		return bottomLowerSandSteps;
	}
	
	/**
     * Gets the number of steps needed for the 
     * sand to sink in the upper part
     * @return upperSandSteps1 the number of steps needed
     */
	public int getUpperSandSteps1()
	{
		return upperSandSteps1;
	}
	
	/**
     * Gets the number of steps needed for the 
     * sand to sink in the upper part
     * @return upperSandSteps2 the number of steps needed
     */
	public int getUpperSandSteps2()
	{
		return upperSandSteps2;
	}
	
	/**
     * Gets the number of steps needed for the 
     * sand to sink in the upper part
     * @return upperSandSteps3 the number of steps needed
     */
	public int getUpperSandSteps3()
	{
		return upperSandSteps3;
	}
	
	/**
     * Gets the number of steps needed for the 
     * sand to sink in the upper part
     * @return upperSandSteps4 the number of steps needed
     */
	public int getUpperSandSteps4()
	{
		return upperSandSteps4;
	}
	
	/**
     * Gets the first value of shrinks for the sand in the upper part
     * @return upperSandShrink1 the value of the first shrink
     */
	public float getUpperSandShrink1()
	{
		return upperSandShrink1;
	}
	
	/**
     * Gets the second value of shrinks for the sand in the upper part
     * @return upperSandShrink2 the value of the second shrink
     */
	public float getUpperSandShrink2()
	{
		return upperSandShrink2;
	}
	
	/**
     * Gets the third value of shrinks for the sand in the upper part
     * @return upperSandShrink3 the value of the third shrink
     */
	public float getUpperSandShrink3()
	{
		return upperSandShrink3;
	}
	
	/**
     * Gets the forth value of shrinks for the sand in the upper part
     * @return upperSandShrink4 the value of the forth shrink
     */
	public float getUpperSandShrink4()
	{
		return upperSandShrink4;
	}
	
	public String toString ( )
	{
		return bulbsTypeName;
	}
	
	public String getBulbsMaterial ( )
	{
		return strBulbsMaterial;
	}
	
	public String getSandMaterial ( )
	{
		return strSandMaterial;
	}
    
	public float getBulbheight ( )
	{
		return bulbheight;
	}
	
	public float getGlassThickness ( )
	{
		return glassThickness;
	}
    
	public float getEndBulbRadius()
	{
		return bRad4;
	}
	
	public float getEndSandRadius()
	{
		return sRad4;
	}
	
	public float getMaxLowerSandLevel()
	{
		return maxLowerSandLevel;
	}
	
	public float getMaxUpperSandLevel()
	{
		return maxUpperSandLevel;
	}
	
	
	// SET-METODER
	
	/**
     * Sets the start radius for the sand heap
     * @param start the start radius
     */
	public void setLowerTinyHeapRadiusStart(float start)
	{
		lowerTinyHeapRadiusStart = start;
	}
	
	/**
     * Sets the number of steps needed for the 
     * lowerTinyHeapRadius to reach the sandEndRadius (sRad4)
     * @param steps the number of steps needed
     */
	public void setLowerTinyHeapSteps (int steps)
	{
		lowerTinyHeapSteps = steps;
	}
	
	/**
     * Sets the number of steps needed for the for the sand up along the walls
     * in the lower part of the glass to go from empty to its max lower level
     * @param steps the number of steps needed
     */
	public void setBottomLowerSandSteps (int steps)
	{
		bottomLowerSandSteps = steps;
	}
	
	/**
     * Sets the value of shrinks for the sand down along the walls
     * in the upper part of the glass
     * @param shrink1 the first value to shrink the sand with
     * @param shrink2 the second value to shrink the sand with
     * @param shrink3 the third value to shrink the sand with
     * @param shrink4 the forth value to shrink the sand with
     */
	public void setUpperSandShrinks (float shrink1, float shrink2, float shrink3, float shrink4)
	{
		upperSandShrink1 = shrink1;
		upperSandShrink2 = shrink2;
		upperSandShrink3 = shrink3;
		upperSandShrink4 = shrink4;
	}
	
	/**
     * Sets the number of steps needed for the sand down along the walls
     * in the upper part of the glass to go from its max upper level to empty
     * @param steps1 the number of steps needed in the first shrink period
     * @param steps2 the number of steps needed in the second shrink period
     * @param steps3 the number of steps needed in the third shrink period
     * @param steps4 the number of steps needed in the forth shrink period
     */
	public void setUpperSandSteps (int steps1, int steps2, int steps3, int steps4)
	{
		upperSandSteps1 = steps1;
		upperSandSteps2 = steps2;
		upperSandSteps3 = steps3;
		upperSandSteps4 = steps4;
	}
	
	public void setBulbsTypeName(String type)
	{
		bulbsTypeName = type;
	}
	
	public void setBulbsMaterial(String strBulbsMaterial)
	{
		this.strBulbsMaterial = strBulbsMaterial;
		bulbsMaterial = matFactory.createMaterial(strBulbsMaterial);
	}
	
	public void setSandMaterial(String strSandMaterial)
	{
		this.strSandMaterial = strSandMaterial;
		sandMaterial = matFactory.createMaterial(strSandMaterial);
	}
		
	public void setBulbheight(float bulbheight)
	{
		this.bulbheight = bulbheight;
	}
	
	public void setAllControlPoints(float ctrz0, float ctrz1, float ctrz2, float ctrz3, float ctrz4,
		float bRad0, float bRad1, float bRad2, float bRad3, float bRad4, 
		float sRad0, float sRad1, float sRad2, float sRad3, float sRad4)
	{
		this.ctrz0 = ctrz0;
		this.ctrz1 = ctrz1;
		this.ctrz2 = ctrz2;
		this.ctrz3 = ctrz3;
		this.ctrz4 = ctrz4;
		this.bRad0 = bRad0;
		this.bRad1 = bRad1;
		this.bRad2 = bRad2;
		this.bRad3 = bRad3;
		this.bRad4 = bRad4;
		this.sRad0 = sRad0;
		this.sRad1 = sRad1;
		this.sRad2 = sRad2;
		this.sRad3 = sRad3;
		this.sRad4 = sRad4;
	}
	
	public void setBulbCtrPoints(float[] bulbCtrPoints)
	{
		this.bulbCtrPoints = bulbCtrPoints;
	}
	
	public void setGlassThickness(float glassThickness)
	{
		this.glassThickness = glassThickness;
	}
	
	public void setMaxLowerSandLevel(float maxLowerSandLevel)
	{
		this.maxLowerSandLevel = maxLowerSandLevel;
	}
	
	public void setMaxUpperSandLevel(float maxUpperSandLevel)
	{
		this.maxUpperSandLevel = maxUpperSandLevel;
	}
	
	public void setSandCtrPoints(float[] sandCtrPoints)
	{
		this.sandCtrPoints = sandCtrPoints;
	}
	
	public void setSandBezierCurvePoints0(float[] sandBezierCurvePoints0)
	{
		this.sandBezierCurvePoints0 = sandBezierCurvePoints0;
	}
	
	public void setSandBezierCurvePoints1(float[] sandBezierCurvePoints1)
	{
		this.sandBezierCurvePoints1 = sandBezierCurvePoints1;
	}
	
	public void setSandBezierCurvePoints2(float[] sandBezierCurvePoints2)
	{
		this.sandBezierCurvePoints2 = sandBezierCurvePoints2;
	}
	
	public void setSandBezierCurvePoints3(float[] sandBezierCurvePoints3)
	{
		this.sandBezierCurvePoints3 = sandBezierCurvePoints3;
	}
	
	public void setSandBezierCurvePoints4(float[] sandBezierCurvePoints4)
	{
		this.sandBezierCurvePoints4 = sandBezierCurvePoints4;
	}
	
	public void testMarkCurve(float pointX, float pointY, float pointZ)
	{
		gl.glPushMatrix();
		gl.glTranslatef(0.0f, -pointY, 0.0f);
		glut.glutSolidSphere (0.1f, 20, 20);
		gl.glPopMatrix();
	}
}