// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum og GLUEnum (gir tilgang til en rekke konstanter (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * An abstract stand to be used togehter with the abstract class <code>Bulbs</code> to make an hourglass.<br>
 * Supplies all necessary common methods for its subclasses.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public abstract class Stand implements GLEnum, GLUEnum
{
	// VARIABLER SOM IKKE FOREKOMMER I SUBKLASSENE
	// Variabler for materialer
	MaterialFactory matFactory; // Brukes til  instansiere riktig klasse av materiale
	Material standMaterial; // Materialet som skal brukes til sttten
	// Representerer et gluNewQuadric object
	long standPart;
	
	// VARIABLER SOM KOMMER SOM PARAMETERE TIL KONSTRUKTREN
	GLFunc gl = null; // Interface for OpenGL "function mapping"
	GLUFunc glu = null; //Interface for GLU "function mapping"
	GLUTFunc glut = null; // Interface for GLUT "function mapping"
	float innerHeight; // Hyden fra midten av glasset til der bunnen/toppen av sttten begynner (tilsvarer bulbHeight
	
	// VARIABLER SOM BLIR SATT VED INITIERING AV STAND (kunne vel ogs kommet som parameter til konstruktren)
	String strStandMaterial; // String versjon av materialet som skal brukes
	
	// VARIABLER SOM INSTANSIERES I SUBKLASSENE
	public float standRadius; //Stttens radius
	String standTypeName; // Stttens type, (Classic, Modern, Bizarre, Simple eller Plain)
	
	/**
	* Stand constructor, used by the classes ClassicStand and BizarreStand.
	* @param gl the <code>GLFunc</code> interface that is used for this component
	* @param glu the <code>GLUFunc</code> interface that is used for this component
	* @param glut the <code>GLUTFunc</code> interface that is used for this component
	* @param bulbheight the height of the bulb, from the middle of the hourglass to the stand
	*/
	public Stand ( GLFunc gl, GLUFunc glu, GLUTFunc glut, float bulbheight ) 
	{
		this.gl = gl;
		this.glu = glu;
		this.glut = glut;
		innerHeight = bulbheight;
		matFactory = new MaterialFactory(gl); // Initierer ny MaterialFactory for  kunne initiere nsket materiale
	}
	
	/**
	* Stand constructor, used by the classes ModernStand, SimpleStand and PlainStand.
	* @param gl the <code>GLFunc</code> interface that is used for this component
	* @param glu the <code>GLUFunc</code> interface that is used for this component
	* @param bulbheight the height of the bulb, from the middle of the hourglass to the stand
	*/
	public Stand ( GLFunc gl, GLUFunc glu, float bulbheight) 
	{
		this.gl = gl;
		this.glu = glu;
		innerHeight = bulbheight;
		matFactory = new MaterialFactory(gl); // Initierer ny MaterialFactory for  kunne initiere nsket materiale
	}
	
	/**
	* Initiates correct material with correct properties and 
	* calls for the two halfs of the hourglass stand to be drawn
	*/
	public void drawStand()
	{
		// Hvis sttten er laget av glass (bare PlainStand brukes til  lage glassbunn)
		// m materialet vre aktivt p begge sider
		if (standTypeName.equals("Plain"))
		{
			standMaterial.enableMaterial(GL_FRONT_AND_BACK); // Aktiviserer nsket materiale		
		}
		// Hvis sttten ikke er av glass trengs materiale bare p forsiden
		else 
		{
			standMaterial.enableMaterial(GL_FRONT); // Aktiviserer nsket materiale
		}
		
		// Tegner sttten 
		standPart = glu.gluNewQuadric(); // Initierer "tegneobjektet"
		gl.glPushMatrix();
		gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f); // Roterer for  kunne tegne nedre halvdel
		drawHalfStand(true); // Tegner den nederste halvdelen
		gl.glRotatef(180.0f, 1.0f, 0.0f, 0.0f); // Roterer for  kunne tegne vre halvdel
		drawHalfStand(false); // Tegner den verste halvdelen
		gl.glPopMatrix();
		glu.gluDeleteQuadric(standPart); // Sletter "tegneobjektet" nr jeg ikke trenger det lenger 
		
		// Pasifiserer brukt materiale 
		standMaterial.disableMaterial(); 
	}
	
	/**
	* Draws a half hourglass stand.<br>
	* All subclasses must implement this method.
	* @param isLowerHalf used if the two halfs need to be different
	*/
	public abstract void drawHalfStand(boolean isLowerHalf);
	
	public void setStandMaterial(String standMat)
	{
		strStandMaterial = standMat;
		standMaterial = matFactory.createMaterial(strStandMaterial); // Initierer nsket materiale
	}
	
	public String toString ( )
	{
		return standTypeName;
	}
	
	/**
	* Returns a string representation of the hourglass stand material. 
	* @return strStandMaterial the material of this component
	*/
	public String getStandMaterial()
	{
		return strStandMaterial;
	}
}